@@ -47,6 +47,7 @@ from mysql.connector import utils, errorcode
 from mysql.connector.constants import (
     ClientFlag, ServerCmd, FieldType, ServerFlag, FieldFlag
 )
+from mysql.connector.errors import InterfaceError
 
 NEXT_CNX_ID = 0
 NEXT_CNX_ID_LOCK = threading.Lock()
@@ -458,10 +459,13 @@ class MySQLRPCRequestHandler(SocketServer.BaseRequestHandler,
         :rtype: tuple
         """
         header = bytearray(b'')
-
-        while len(header) < 4:
-            chunk = self.request.recv(4)
+        header_len = 0
+        while header_len < 4:
+            chunk = self.request.recv(4 - header_len)
+            if not chunk:
+                raise InterfaceError(errno=2013)
             header += chunk
+            header_len = len(header)
 
         length = struct.unpack_from('<I', buffer(header[0:3] + b'\x00'))[0]
         self._curr_pktnr = header[-1]
@@ -1108,7 +1112,9 @@ class MySQLRPCServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
             value = config.get('protocol.mysql', 'disable_authentication')
             if value.lower() == 'yes':
                 self.__auth_disabled = True
-                _LOGGER.warning("Authentication disabled")
+                _LOGGER.warning(
+                    "Authentication disabled for MySQL RPC protocol."
+                )
         except:
             self.__auth_disabled = False
         else:
