@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ This file contains the reporting mechanisms for reporting disk usage.
 """
 import getpass
 import os
+import shlex
 import subprocess
 import sys
 import tempfile
@@ -185,9 +186,13 @@ def _server_info(server_val, get_defaults=False, options=None):
             if log_file not in ["stderr", "stdout"]:
                 # Now get the information about the size of the logs
                 try:
+                    # log_file might be a relative path, in which case we need
+                    # to prepend the datadir path to it
+                    if not os.path.isabs(log_file):
+                        log_file = os.path.join(params_dict['datadir'],
+                                                log_file)
                     params_dict[log_tpl.log_file_size] = "{0} bytes".format(
                         os.path.getsize(log_file))
-
                 except os.error:
                     # if we are unable to get the log_file_size
                     params_dict[log_tpl.log_file_size] = ''
@@ -247,7 +252,7 @@ def _server_info(server_val, get_defaults=False, options=None):
         if server.is_alias('localhost'):
             try:
                 my_def_path = get_tool_path(params_dict['basedir'],
-                                            "my_print_defaults")
+                                            "my_print_defaults", quote=True)
             except UtilError as err:
                 raise UtilError("Unable to retrieve the defaults data "
                                 "(requires access to my_print_defaults): {0} "
@@ -256,7 +261,9 @@ def _server_info(server_val, get_defaults=False, options=None):
                                 )
             out_file = tempfile.TemporaryFile()
             # Execute tool: <basedir>/my_print_defaults mysqld
-            subprocess.call([my_def_path, "mysqld"], stdout=out_file)
+            cmd_list = shlex.split(my_def_path)
+            cmd_list.append("mysqld")
+            subprocess.call(cmd_list, stdout=out_file)
             out_file.seek(0)
             # Get defaults data from temp output file.
             defaults.append("\nDefaults for server {0}".format(server_id))
@@ -306,7 +313,7 @@ def _start_server(server_val, basedir, datadir, options=None):
     verbosity = options.get("verbosity", 0)
     start_timeout = options.get("start_timeout", 10)
 
-    mysqld_path = get_tool_path(basedir, "mysqld")
+    mysqld_path = get_tool_path(basedir, "mysqld", quote=True)
 
     print "# Server is offline."
 
@@ -334,7 +341,8 @@ def _start_server(server_val, basedir, datadir, options=None):
         print "# --- BEGIN (server output) ---"
     else:
         print "# Starting read-only instance of the server ...",
-    args = [
+    args = shlex.split(mysqld_path)
+    args.extend([
         "--no-defaults",
         "--skip-grant-tables",
         "--read_only",
@@ -342,7 +350,7 @@ def _start_server(server_val, basedir, datadir, options=None):
         "--basedir=" + basedir,
         "--datadir=" + datadir,
         "--user={0}".format(user_name),
-    ]
+    ])
 
     # It the server is 5.6 or later, we must use additional parameters
     if post_5_5:
@@ -356,7 +364,6 @@ def _start_server(server_val, basedir, datadir, options=None):
         if not post_5_7_4:
             server_args.append("--skip-innodb")
         args.extend(server_args)
-    args.insert(0, mysqld_path)
 
     socket = server_val.get('unix_socket', None)
     if socket is not None:
@@ -395,8 +402,9 @@ def _start_server(server_val, basedir, datadir, options=None):
 
     # Raise last known exception (if unable to connect to the server)
     if error:
-        raise error  # pylint: disable=E0702
-                     # See: http://www.logilab.org/ticket/3207
+        # See: http://www.logilab.org/ticket/3207
+        # pylint: disable=E0702
+        raise error
 
     if verbosity > 0:
         print "# done (server started)."
@@ -422,7 +430,7 @@ def _stop_server(server_val, basedir, options=None):
         options = {}
     verbosity = options.get("verbosity", 0)
     socket = server_val.get("unix_socket", None)
-    mysqladmin_path = get_tool_path(basedir, "mysqladmin")
+    mysqladmin_path = get_tool_path(basedir, "mysqladmin", quote=True)
 
     # Stop the instance
     if verbosity > 0:
@@ -522,6 +530,7 @@ def show_server_info(servers, options):
     ssl_dict['ssl_cert'] = options.get("ssl_cert", None)
     ssl_dict['ssl_ca'] = options.get("ssl_ca", None)
     ssl_dict['ssl_key'] = options.get("ssl_key", None)
+    ssl_dict['ssl'] = options.get("ssl", None)
 
     row_dict_lst = []
     warnings = []
