@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -261,9 +261,9 @@ class Database(object):
             'fetch': False,
             'commit': False,  # No COMMIT needed for DDL operations (default).
         }
-        self.constraints = deque()  # Used to store constraints to execute
-                                    # after table creation, deque is
-                                    # thread-safe
+        # Used to store constraints to execute after table creation, deque is
+        # thread-safe
+        self.constraints = deque()
 
         self.objects = []
         self.new_objects = []
@@ -654,6 +654,10 @@ class Database(object):
             pass
         for stm in create_list:
             try:
+                if obj_type == _GRANT:
+                    user = User(self.destination, obj[0])
+                    if not user.exists():
+                        user.create()
                 self.destination.exec_query(stm, self.query_options)
             except Exception as e:
                 raise UtilDBError("Cannot operate on {0} object."
@@ -777,7 +781,7 @@ class Database(object):
             try:
                 query = self.constraints.pop()
             except IndexError:
-                #queue is empty, exit while statement
+                # queue is empty, exit while statement
                 break
             if self.verbose:
                 print(query)
@@ -1373,16 +1377,26 @@ class Database(object):
                 TABLES.TABLE_SCHEMA = REFERENTIAL_CONSTRAINTS.CONSTRAINT_SCHEMA
                 AND
                 TABLES.TABLE_NAME = REFERENTIAL_CONSTRAINTS.TABLE_NAME
-            LEFT JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE ON
+            LEFT JOIN (
+                  SELECT CONSTRAINT_SCHEMA, TABLE_NAME, CONSTRAINT_NAME,
+                         GROUP_CONCAT(COLUMN_NAME) AS COLUMN_NAME,
+                         REFERENCED_TABLE_SCHEMA, REFERENCED_COLUMN_NAME
+                  FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
+                  GROUP BY CONSTRAINT_SCHEMA, TABLE_NAME, CONSTRAINT_NAME,
+                           REFERENCED_TABLE_SCHEMA, REFERENCED_COLUMN_NAME
+            ) AS KEY_COLUMN_USAGE ON
                 TABLES.TABLE_SCHEMA = KEY_COLUMN_USAGE.CONSTRAINT_SCHEMA
                 AND
                 TABLES.TABLE_NAME = KEY_COLUMN_USAGE.TABLE_NAME
             WHERE TABLES.TABLE_SCHEMA = '%s' AND TABLE_TYPE <> 'VIEW' %s
-            GROUP BY TABLES.TABLE_SCHEMA, TABLES.TABLE_NAME,
-                     COLUMNS.ORDINAL_POSITION
+            """
+            _ORDER_BY_DEFAULT = """
             ORDER BY TABLES.TABLE_SCHEMA, TABLES.TABLE_NAME,
                      COLUMNS.ORDINAL_POSITION
             """
+            _ORDER_BY_NAME = """
+            ORDER BY TABLES.TABLE_NAME
+            """
             exclude_param = "TABLES.TABLE_NAME"
 
         elif obj_type == _VIEW:
@@ -1406,6 +1420,8 @@ class Database(object):
             FROM INFORMATION_SCHEMA.VIEWS
             WHERE TABLE_SCHEMA = '%s' %s
             """
+            _ORDER_BY_DEFAULT = ""
+            _ORDER_BY_NAME = ""
             exclude_param = "VIEWS.TABLE_NAME"
         elif obj_type == _TRIG:
             _NAMES = """
@@ -1438,6 +1454,8 @@ class Database(object):
             FROM INFORMATION_SCHEMA.TRIGGERS
             WHERE TRIGGER_SCHEMA = '%s' %s
             """
+            _ORDER_BY_DEFAULT = ""
+            _ORDER_BY_NAME = ""
             exclude_param = "TRIGGERS.TRIGGER_NAME"
         elif obj_type == _PROC:
             _NAMES = """
@@ -1464,6 +1482,8 @@ class Database(object):
             FROM mysql.proc
             WHERE DB = '%s' AND TYPE = 'PROCEDURE' %s
             """
+            _ORDER_BY_DEFAULT = ""
+            _ORDER_BY_NAME = ""
             exclude_param = "NAME"
         elif obj_type == _FUNC:
             _NAMES = """
@@ -1490,6 +1510,8 @@ class Database(object):
             FROM mysql.proc
             WHERE DB = '%s' AND TYPE = 'FUNCTION' %s
             """
+            _ORDER_BY_DEFAULT = ""
+            _ORDER_BY_NAME = ""
             exclude_param = "NAME"
         elif obj_type == _EVENT:
             _NAMES = """
@@ -1516,6 +1538,8 @@ class Database(object):
             FROM mysql.event
             WHERE DB = '%s' %s
             """
+            _ORDER_BY_DEFAULT = ""
+            _ORDER_BY_NAME = ""
             exclude_param = "NAME"
         elif obj_type == _GRANT:
             _OBJECT_QUERY = """
@@ -1558,19 +1582,23 @@ class Database(object):
                 prefix = _NAMES
                 if need_backtick:
                     pos_to_quote = names_pos_to_quote
+                sufix = _ORDER_BY_NAME
             elif columns == "full":
                 prefix = _FULL
                 if need_backtick:
                     pos_to_quote = full_pos_to_quote
+                sufix = _ORDER_BY_DEFAULT
             else:
                 prefix = _MINIMAL
                 if need_backtick:
                     pos_to_quote = minimal_pos_to_quote
+                sufix = _ORDER_BY_DEFAULT
             # Form exclusion string
             exclude_str = ""
             if self.exclude_patterns:
                 exclude_str = self.__build_exclude_patterns(exclude_param)
-            query = prefix + _OBJECT_QUERY % (self.db_name, exclude_str)
+            query = (prefix + _OBJECT_QUERY + sufix) % (self.db_name,
+                                                        exclude_str)
             res = self.source.exec_query(query, col_options)
 
             # Quote required identifiers with backticks
@@ -1581,7 +1609,7 @@ class Database(object):
                 new_rows = []
                 for row in res[1]:
                     # recreate row tuple quoting needed elements with backticks
-                    #r = tuple([quote(i, data) for i, data in enumerate(row)])
+                    # r = tuple([quote(i, data) for i, data in enumerate(row)])
                     r = tuple([quote_with_backticks(data)
                                if data and i in pos_to_quote else data
                                for i, data in enumerate(row)])
