@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -317,6 +317,7 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
     ssl_ca = None
     ssl_cert = None
     ssl_key = None
+    ssl = None
 
     # Split on the '@' to determine the connection string format.
     # The user/password may have the '@' character, split by last occurrence.
@@ -479,6 +480,7 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
                 ssl_ca = config_path_data.get('ssl-ca', None)
                 ssl_cert = config_path_data.get('ssl-cert', None)
                 ssl_key = config_path_data.get('ssl-key', None)
+                ssl = config_path_data.get('ssl', None)
 
         else:
             if login_path and not config_path:
@@ -534,10 +536,11 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
     if isinstance(options, dict):
         charset = options.get("charset", None)
         # If one SSL option was found before, not mix with those in options.
-        if not ssl_cert and not ssl_ca and not ssl_key:
+        if not ssl_cert and not ssl_ca and not ssl_key and not ssl:
             ssl_cert = options.get("ssl_cert", None)
             ssl_ca = options.get("ssl_ca", None)
             ssl_key = options.get("ssl_key", None)
+            ssl = options.get("ssl", None)
 
     else:
         # options is an instance of optparse.Values
@@ -546,7 +549,7 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
         except AttributeError:
             charset = None
         # If one SSL option was found before, not mix with those in options.
-        if not ssl_cert and not ssl_ca and not ssl_key:
+        if not ssl_cert and not ssl_ca and not ssl_key and not ssl:
             try:
                 ssl_cert = options.ssl_cert  # pylint: disable=E1103
             except AttributeError:
@@ -559,6 +562,10 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
                 ssl_key = options.ssl_key  # pylint: disable=E1103
             except AttributeError:
                 ssl_key = None
+            try:
+                ssl = options.ssl  # pylint: disable=E1103
+            except AttributeError:
+                ssl = None
 
     # Set parsed connection values
     connection = {
@@ -576,6 +583,8 @@ def parse_connection(connection_values, my_defaults_reader=None, options=None):
         connection["ssl_ca"] = ssl_ca
     if ssl_key:
         connection["ssl_key"] = ssl_key
+    if ssl:
+        connection["ssl"] = ssl
     # Handle optional parameters. They are only stored in the dict if
     # they were provided in the specifier.
     if socket is not None and os.name == "posix":
@@ -648,7 +657,7 @@ def _verify_parsing(connection_str, host, port, socket, address_type,
     parsed_connection_list = []
     if host:
         log.debug("host {0}".format(host))
-        if address_type == ipv6 and not "[" in connection_str:
+        if address_type == ipv6 and "[" not in connection_str:
             host = host.replace("[", "")
             host = host.replace("]", "")
         parsed_connection_list.append(host)
@@ -707,11 +716,35 @@ def format_IPv6(host_address):
     """Format IPv6 host address
     """
     if host_address:
-        if not "]" in host_address:
+        if "]" not in host_address:
             host_address = "[{0}]".format(host_address)
     return host_address
 
 
+def parse_login_values_config_path(login_values, quietly=True):
+    """Parse the login values to retrieve the user and password from a
+    configuration file.
+
+    login_values[in]    The login values to be parsed.
+    quietly[in]         Do not raise exceptions (Default True).
+
+    returns parsed (user, password) tuple or (login_values, None) tuple.
+    """
+    try:
+        matches = _match(_CONN_CONFIGPATH, login_values, trow_error=False)
+        if matches:
+            path = matches[0]
+            group = matches[1]
+            data = handle_config_path(path, group, use_default=False)
+            user = data.get('user', None)
+            passwd = data.get('password', None)
+            return user, passwd
+    except (FormatError, UtilError):
+        if not quietly:
+            raise
+    return login_values, None
+
+
 def find_password(value):
     """Search for password in a string
 
@@ -720,7 +753,7 @@ def find_password(value):
     if not type(value) == str:
         return False
     # has to have an @ sign
-    if not '@' in value:
+    if '@' not in value:
         return False
     match = _CONN_USERPASS.match(value)
     if not match:
