@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ _MASTER_INFO_COL = [
     'Master_SSL_CA_File', 'Master_SSL_CA_Path', 'Master_SSL_Cert',
     'Master_SSL_Cipher', 'Master_SSL_Key', 'Master_SSL_Verify_Server_Cert',
     'Heartbeat', 'Bind', 'Ignored_server_ids', 'Uuid', 'Retry_count',
-    'SSL_CRL', 'SSL_CRL_Path', 'Enabled_auto_position',
+    'SSL_CRL', 'SSL_CRL_Path', 'Enabled_auto_position', 'Channel_Name',
 ]
 
 _SLAVE_IO_STATE, _SLAVE_MASTER_HOST, _SLAVE_MASTER_USER, _SLAVE_MASTER_PORT, \
@@ -60,6 +60,12 @@ _RPL_USER_QUERY = """
     FROM mysql.user
     WHERE repl_slave_priv = 'Y'
 """
+# Query for server versions >= 5.7.6.
+_RPL_USER_QUERY_5_7_6 = """
+    SELECT user, host, authentication_string = "" as has_password
+    FROM mysql.user
+    WHERE repl_slave_priv = 'Y'
+"""
 
 _WARNING = "# WARNING: %s"
 _MASTER_BINLOG = "Server '%s' does not have binary logging turned on."
@@ -85,125 +91,6 @@ def _get_list(rows, cols):
     return ostream.getvalue().splitlines()
 
 
-def get_last_server_gtid(gtid_set, server_uuid):
-    """Get the last GTID of the specified GTID set for the given server UUID.
-
-    This function retrieves the last GTID from the specified set for the
-    specified server UUID. In more detail, it returns the GTID with the greater
-    sequence value that matches the specified UUID.
-
-    Note: The method assumes that GTID sets are grouped by UUID (separated by
-    comma ',') and intervals appear in ascending order (i.e., the last one is
-    the greater one).
-
-    gtid_set[in]        GTID set to search and get last (greater) GTID value.
-    server_uuid[in]     Server UUID to match, as a GTID set might contain data
-                        for different servers (UUIDs).
-
-    Returns a string with the last GTID value in the set for the given server
-    UUID in the format 'uuid:n'. If no GTID are found in the set for the
-    specified server UUID then None is returned.
-    """
-    uuid_sets = gtid_set.split(',')
-    for uuid_set in uuid_sets:
-        uuid_set_elements = uuid_set.strip().split(':')
-        # Note: UUID are case insensitive, but can appear with mixed cases for
-        # some server versions (e.g., for 5.6.9, lower case in server_id
-        # variable and upper case in GTID_EXECUTED set).
-        if uuid_set_elements[0].lower() == server_uuid.lower():
-            last_interval = uuid_set_elements[-1]
-            try:
-                _, end_val = last_interval.split('-')
-                return '{0}:{1}'.format(server_uuid, end_val)
-            except ValueError:
-                # Error raised for single values (not an interval).
-                return '{0}:{1}'.format(server_uuid, last_interval)
-
-        else:
-            # Skip uuid_set if it does not match the current server uuid.
-            continue
-    return None
-
-
-def gtid_set_cardinality(gtid_set):
-    """Determine the cardinality of the specified GTID set.
-
-    This function counts the number of elements in the specified GTID set.
-
-    gtid_set[in]    target set of GTIDs to determine the cardinality.
-
-    Returns the number of elements of the specified GTID set.
-    """
-    count = 0
-    uuid_sets = gtid_set.split(',')
-    for uuid_set in uuid_sets:
-        intervals = uuid_set.strip().split(':')[1:]
-        for interval in intervals:
-            try:
-                start_val, end_val = interval.split('-')
-                count = int(end_val) - int(start_val) + 1
-            except ValueError:
-                # Error raised for single values (not an interval).
-                count += 1
-    return count
-
-
-def gtid_set_union(gtid_set_a, gtid_set_b):
-    """Perform the union of two GTID sets.
-
-    This method computes the union of two GTID sets and returns the result of
-    the operation.
-
-    gtid_set_a[in]      First GTID set (set A).
-    gtid_set_b[in]      Second GTID set (set B).
-
-    Returns a string with the result of the set union operation between the
-    two given GTID sets.
-    """
-    a_values = gtid_set_a.split(':')
-    b_values = gtid_set_b.split(':')
-    uuid_a = a_values[0]
-    uuid_b = b_values[0]
-    if uuid_a != uuid_b:
-        # Return the concatenation of both sets if the UUID is different.
-        return "{0},{1}".format(gtid_set_a, gtid_set_b)
-    else:
-        # Only perform the union of the intervals if the UUID is the same.
-        # Convert the intervals strings to a single list of tuples with
-        # integers, in order to be handled easily.
-        intervals_list = []
-        for values in a_values[1:]:
-            interval = values.split('-')
-            intervals_list.append((int(interval[0]), int(interval[-1])))
-        for values in b_values[1:]:
-            interval = values.split('-')
-            intervals_list.append((int(interval[0]), int(interval[-1])))
-        # Compute the union of the tuples (intervals).
-        union_set = []
-        for start, end in sorted(intervals_list):
-            # Note: no interval start before the next one (ordered list).
-            if union_set and start <= union_set[-1][1] + 1:
-                # Current interval intersects or is consecutive to the last
-                # one in the results.
-                if union_set[-1][1] < end:
-                    # If the end of the interval is greater than the last one
-                    # then augment it (set the new end), otherwise do nothing
-                    # (meaning the interval is fully included in the last one).
-                    union_set[-1] = (union_set[-1][0], end)
-            else:
-                # No interval in the results or the interval does not intersect
-                # nor is consecutive to the last one, then add it to the end of
-                # the results list.
-                union_set.append((start, end))
-        # Convert resulting union set to a valid string format.
-        union_str = ":".join(
-            ["{0}-{1}".format(vals[0], vals[1])
-             if vals[0] != vals[1] else str(vals[0]) for vals in union_set]
-        )
-        # Concatenate UUID and return the result.
-        return "{0}:{1}".format(uuid_a, union_str)
-
-
 def negotiate_rpl_connection(server, is_master=True, strict=True,
                              options=None):
     """Determine replication connection
@@ -384,8 +271,9 @@ class Replication(object):
         self.ssl_ca = options.get("ssl_ca", None)
         self.ssl_cert = options.get("ssl_cert", None)
         self.ssl_key = options.get("ssl_key", None)
+        self.ssl_opt = options.get("ssl", None)
         self.ssl = False
-        if self.ssl_ca or self.ssl_cert or self.ssl_key:
+        if self.ssl_ca or self.ssl_cert or self.ssl_key or self.ssl_opt:
             self.ssl = True
         self.master = master
         self.slave = slave
@@ -807,7 +695,7 @@ class Replication(object):
             elif not sql_running:
                 if self.verbosity > 0:
                     print "# Retry to start the slave SQL thread..."
-                #SQL thread is not running, retry to start it
+                # SQL thread is not running, retry to start it
                 res = self.slave.start_sql_thread(self.query_options)
             if self.verbosity > 0:
                 print "# Waiting for slave to synchronize with master"
@@ -944,7 +832,12 @@ class Master(Server):
         """
         if options is None:
             options = {}
-        return self.exec_query(_RPL_USER_QUERY, options)
+        # Use the correct query for server (changed for 5.7.6).
+        if self.check_version_compat(5, 7, 6):
+            query = _RPL_USER_QUERY_5_7_6
+        else:
+            query = _RPL_USER_QUERY
+        return self.exec_query(query, options)
 
     def create_rpl_user(self, host, port, r_user, r_pass=None, verbosity=0,
                         ssl=False):
@@ -965,11 +858,17 @@ class Master(Server):
                                     (False, <error>) = error
         """
 
+        grants_enabled = self.grant_tables_enabled()
+        if not grants_enabled:
+            return (True, None)
+
         if "]" in host:
             host = clean_IPv6(host)
 
         # Create user class instance
-        user = User(self, "%s@%s:%s" % (r_user, host, port), verbosity)
+        user = User(self, "%s:%s@%s:%s" % (r_user, r_pass, host, port))
+        if not user.exists():
+            user.create()
 
         if not user.has_privilege("*", "*", "REPLICATION SLAVE"):
             if verbosity > 0:
@@ -1091,7 +990,8 @@ class Master(Server):
                                   'host': row[1], 'port': row[2],
                                   'socket': None, 'ssl_ca': self.ssl_ca,
                                   'ssl_cert': self.ssl_cert,
-                                  'ssl_key': self.ssl_key},
+                                  'ssl_key': self.ssl_key,
+                                  'ssl': self.ssl},
                     'role': 'slave',
                     'verbose': verbose,
                 }
@@ -1278,9 +1178,14 @@ class MasterInfo(object):
         if res is None or res == []:
             return False
 
+        # Protect overrun of array if the master_info table size has changed
+        # (more rows than expected).
+        num = len(res[0][1:])
+        if num > len(_MASTER_INFO_COL):
+            num = len(_MASTER_INFO_COL)
         # Build dictionary for the information with column information
         rows = []
-        for i in range(0, len(res[0][1:])):
+        for i in range(0, num):
             rows.append(res[0][i + 1])
         self._build_dictionary(rows)
 
@@ -1481,7 +1386,8 @@ class Slave(Server):
     def is_connected(self):
         """Check to see if slave is connected to master
 
-        This method will check the slave to see if it is connected to a master.
+        This method will check the slave to see if it is connected to a master
+        by checking if his I/O Thread is running.
 
         Returns bool - True = slave is connected
         """
@@ -1871,7 +1777,7 @@ class Slave(Server):
         """
         slave_gtids = self.exec_query(_GTID_EXECUTED)[0][0]
         gtids = self.exec_query("SELECT GTID_SUBTRACT('%s','%s')" %
-                               (master_gtids[0][0], slave_gtids))[0]
+                                (master_gtids[0][0], slave_gtids))[0]
         # Init gtid_behind count (if no GTIDs behind then 0 is returned)
         gtid_behind = 0
         # Check if there are GTIDs behind
@@ -1990,6 +1896,8 @@ class Slave(Server):
             master_ssl_ca = master_values.get('Master_SSL_CA_File', None)
             master_ssl_cert = master_values.get('Master_SSL_Cert', None)
             master_ssl_key = master_values.get('Master_SSL_Key', None)
+            if master_ssl and master_ssl_ca is None:
+                master_ssl_ca = ''
         else:
             master_host = master_values.get('Master_Host',
                                             master_info['Master_Host'])
@@ -2030,9 +1938,9 @@ class Slave(Server):
         if master_passwd is not None:
             change_master += "MASTER_PASSWORD = '%s', " % master_passwd
         change_master += "MASTER_PORT = %s" % master_port
-        if master_ssl:
+        if master_ssl and master_ssl not in ('0', 'OFF'):
             change_master = "{0}, MASTER_SSL = {1}".format(change_master, 1)
-        if master_ssl_ca:
+        if master_ssl_ca is not None:
             change_master = ("{0}, MASTER_SSL_CA = '{1}'"
                              ).format(change_master, master_ssl_ca)
         if master_ssl_cert:
@@ -2066,7 +1974,7 @@ class Slave(Server):
         res = self.get_status()
         if res == [] or not res[0]:
             if raise_error:
-                raise UtilRplError("Server '{0}:{1}' is not acting as a Slave "
+                raise UtilRplError("Server '{0}:{1}' is not acting as a slave "
                                    "(slave status is empty)"
                                    ".".format(self.host, self.port))
             return False
