@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ import string
 import subprocess
 import tempfile
 import threading
+import logging
 
 import mysql.connector
 from mysql.connector.constants import ClientFlag
@@ -40,6 +41,7 @@ from mysql.utilities.common.tools import (delete_directory, execute_script,
                                           ping_host)
 from mysql.utilities.common.ip_parser import (parse_connection, hostname_is_ip,
                                               clean_IPv6, format_IPv6)
+from mysql.utilities.common.messages import MSG_MYSQL_VERSION
 
 
 _FOREIGN_KEY_SET = "SET foreign_key_checks = {0}"
@@ -126,7 +128,8 @@ def get_connection_dictionary(conn_info, ssl_dict=None):
         if (ssl_dict is not None and
             not (conn_info.get("ssl_ca", None) or
                  conn_info.get("ssl_cert", None) or
-                 conn_info.get("ssl_key", None))):
+                 conn_info.get("ssl_key", None) or
+                 conn_info.get("ssl", None))):
             conn_info.update(ssl_dict)
         conn_val = conn_info
     elif isinstance(conn_info, Server):
@@ -142,6 +145,53 @@ def get_connection_dictionary(conn_info, ssl_dict=None):
     return conn_val
 
 
+def set_ssl_opts_in_connection_info(ssl_opts, connection_info):
+    """Sets the ssl options in a connection information to be used with C/py.
+
+    ssl_opts[in]           A dictionary with the ssl options (ssl_ca, ssl_cert
+                           and ssl_key).
+    connection_info[out]   A dictionary to set the ssl options after validate
+                           them.
+
+    The ssl options will be set on the connection_info if they are not None.
+    In addition the SSL client flags are added if at least one ssl option is
+    set.
+    """
+    # Add SSL parameters ONLY if they are not None
+    add_ssl_flag = False
+    if ssl_opts.get('ssl_ca') is not None:
+        connection_info['ssl_ca'] = ssl_opts.get('ssl_ca')
+        add_ssl_flag = True
+    if ssl_opts.get('ssl_cert') is not None:
+        connection_info['ssl_cert'] = ssl_opts.get('ssl_cert')
+        add_ssl_flag = True
+    if ssl_opts.get('ssl_key') is not None:
+        connection_info['ssl_key'] = ssl_opts.get('ssl_key')
+        add_ssl_flag = True
+    if ssl_opts.get('ssl'):
+        add_ssl_flag = True
+
+    # When at least one of cert, key or ssl options are specified, the ca
+    # option is not required for establishing the encrypted connection,
+    # but C/py will not allow the None value for the ca option, so we use an
+    # empty string i.e '' to avoid an error from C/py about ca option being
+    # the None value.
+    if ('ssl_cert' in connection_info.keys() or
+        'ssl_key' in connection_info.keys() or
+        ssl_opts.get('ssl')) and \
+       'ssl_ca' not in connection_info.keys():
+        connection_info['ssl_ca'] = ''
+
+    # The ca certificate is verified only if the ssl option is also specified.
+    if ssl_opts.get('ssl') and connection_info['ssl_ca']:
+        connection_info['ssl_verify_cert'] = True
+
+    if add_ssl_flag:
+        cpy_flags = [ClientFlag.SSL,
+                     ClientFlag.SSL_VERIFY_SERVER_CERT]
+        connection_info['client_flags'] = cpy_flags
+
+
 def _print_connection(prefix, conn_info):
     """Print connection information
 
@@ -233,7 +283,7 @@ def get_local_servers(all_proc=False, start=3306, end=3333,
                 # Look for port on either local or foreign address
                 port = proginfo[1][proginfo[1].find(":") + 1:]
                 if proginfo[1][0] == '0' and port.isdigit():
-                    if int(port) >= start and int(port) <= end:
+                    if int(port) >= int(start) and int(port) <= int(end):
                         processes.append((proginfo[4], port))
                         break
                 if len(proginfo) > 2:
@@ -395,6 +445,8 @@ def connect_servers(src_val, dest_val, options=None):
         ssl_dict['ssl_ca'] = options.get("ssl_ca", None)
     if options.get("ssl_key", None) is not None:
         ssl_dict['ssl_key'] = options.get("ssl_key", None)
+    if options.get("ssl", None) is not None:
+        ssl_dict['ssl'] = options.get("ssl", None)
 
     source = None
     destination = None
@@ -539,7 +591,6 @@ def stop_running_server(server, wait=10, drop=True):
         return True
 
     # Build the shutdown command
-    cmd = ""
     res = server.show_server_variable("basedir")
     mysqladmin_client = "mysqladmin"
     if not os.name == "posix":
@@ -557,10 +608,14 @@ def stop_running_server(server, wait=10, drop=True):
         mysqladmin_path = os.path.normpath(os.path.join(res[0][1],
                                                         "client/release",
                                                         mysqladmin_client))
-    cmd += mysqladmin_path
+    if os.name == 'posix':
+        cmd = "'{0}'".format(mysqladmin_path)
+    else:
+        cmd = '"{0}"'.format(mysqladmin_path)
     if server.socket is None and server.host == 'localhost':
         server.host = '127.0.0.1'
-    cmd += " shutdown --user={0} --host={1} ".format(server.user, server.host)
+    cmd = "{0} shutdown --user={1} --host={2} ".format(cmd, server.user,
+                                                       server.host)
     if server.passwd:
         cmd = "{0} --password={1} ".format(cmd, server.passwd)
     # Use of server socket only works with 'localhost' (not with 127.0.0.1).
@@ -577,7 +632,7 @@ def stop_running_server(server, wait=10, drop=True):
             cmd = "{0} --ssl-key={1} ".format(cmd, server.ssl_key)
 
     res = server.show_server_variable("datadir")
-    datadir = res[0][1]
+    datadir = os.path.normpath(res[0][1])
 
     # Kill all connections so shutdown will work correctly
     res = server.exec_query("SHOW PROCESSLIST")
@@ -616,6 +671,22 @@ def stop_running_server(server, wait=10, drop=True):
     return True
 
 
+def log_server_version(server, level=logging.INFO):
+    """Log server version message.
+
+    This method will log the server version message.
+    If no log file is provided it will also print the message to stdout.
+
+    server[in]           Server instance.
+    level[in]            Level of message to log. Default = INFO.
+    print_version[in]    If True, print the message to stdout. Default = True.
+    """
+    host_port = "{host}:{port}".format(**get_connection_dictionary(server))
+    version_msg = MSG_MYSQL_VERSION.format(server=host_port,
+                                           version=server.get_version())
+    logging.log(level, version_msg)
+
+
 class Server(object):
     """The Server class can be used to connect to a running MySQL server.
     The following utilities are provided:
@@ -677,7 +748,8 @@ class Server(object):
             self.ssl_ca = conn_values.get('ssl_ca', None)
             self.ssl_cert = conn_values.get('ssl_cert', None)
             self.ssl_key = conn_values.get('ssl_key', None)
-            if self.ssl_cert or self.ssl_ca or self.ssl_key:
+            self.ssl = conn_values.get('ssl', False)
+            if self.ssl_cert or self.ssl_ca or self.ssl_key or self.ssl:
                 self.has_ssl = True
         except KeyError:
             raise UtilError("Dictionary format not recognized.")
@@ -960,27 +1032,37 @@ class Server(object):
             conn_vals["ssl_cert"] = self.ssl_cert
         if self.ssl_key:
             conn_vals["ssl_key"] = self.ssl_key
+        if self.ssl:
+            conn_vals["ssl"] = self.ssl
 
         return conn_vals
 
-    def connect(self):
+    def connect(self, log_version=False):
         """Connect to server
 
         Attempts to connect to the server as specified by the connection
         parameters.
 
+        log_version[in]      If True, log server version. Default = False.
+
         Note: This method must be called before executing queries.
 
         Raises UtilError if error during connect
         """
         try:
             self.db_conn = self.get_connection()
+            if log_version:
+                log_server_version(self)
             # If no charset provided, get it from the "character_set_client"
             # server variable.
             if not self.charset:
                 res = self.show_server_variable('character_set_client')
                 self.db_conn.set_charset_collation(charset=res[0][1])
                 self.charset = res[0][1]
+            if self.ssl:
+                res = self.exec_query("SHOW STATUS LIKE 'Ssl_cipher'")
+                if res[0][1] == '':
+                    raise UtilError("Can not encrypt server connection.")
         except UtilError:
             # Reset any previous value if the connection cannot be established,
             # before raising an exception. This prevents the use of a broken
@@ -1021,6 +1103,23 @@ class Server(object):
                 parameters['ssl_cert'] = self.ssl_cert
             if self.ssl_key is not None:
                 parameters['ssl_key'] = self.ssl_key
+
+            # When at least one of cert, key or ssl options are specified,
+            # the ca option is not required for establishing the encrypted
+            # connection, but C/py will not allow the None value for the ca
+            # option, so we use an empty string i.e '' to avoid an error from
+            # C/py about ca option being the None value.
+            if ('ssl_cert' in parameters.keys() or
+                'ssl_key' in parameters.keys() or
+                self.ssl) and \
+               'ssl_ca' not in parameters:
+                parameters['ssl_ca'] = ''
+
+            # The ca certificate is verified only if the ssl option is also
+            # specified.
+            if self.ssl and parameters['ssl_ca']:
+                parameters['ssl_verify_cert'] = True
+
             if self.has_ssl:
                 cpy_flags = [ClientFlag.SSL, ClientFlag.SSL_VERIFY_SERVER_CERT]
                 parameters['client_flags'] = cpy_flags
@@ -1238,6 +1337,53 @@ class Server(object):
 
         return self.exec_query("SHOW VARIABLES LIKE '%s'" % variable)
 
+    def select_variable(self, var_name, var_type=None):
+        """Get server system variable value using SELECT statement.
+
+        This function displays the value of system variables using the SELECT
+        statement. This can be used as a workaround for variables with very
+        long values, as SHOW VARIABLES is subject to a version-dependent
+        display-width limit.
+
+        Note: Some variables may not be available using SELECT @@var_name, in
+        such cases use SHOW VARIABLES LIKE 'var_name'.
+
+        var_name[in]    Name of the variable to display.
+        var_type[in]    Type of the variable ('session' or 'global'). By
+                        default no type is used, meaning that the session
+                        value is returned if it exists and the global value
+                        otherwise.
+
+        Return the value for the given server system variable.
+        """
+        if var_type is None:
+            var_type = ''
+        elif var_type.lower() in ('global', 'session', ''):
+            var_type = '{0}.'.format(var_type)  # Add dot (.)
+        else:
+            raise UtilDBError("Invalid variable type: {0}. Supported types: "
+                              "'global' and 'session'.".format(var_type))
+        # Execute SELECT @@[var_type.]var_name.
+        # Note: An error is issued if the given variable is not known.
+        res = self.exec_query("SELECT @@{0}{1}".format(var_type, var_name))
+        return res[0][0]
+
+    def flush_logs(self, log_type=None):
+        """Execute the FLUSH [log_type] LOGS statement.
+
+        Reload internal logs cache and closes and reopens all log files, or
+        only of the specified log_type.
+
+        Note: The log_type option is available from MySQL 5.5.3.
+
+        log_type[in]    Type of the log files to be flushed. Supported values:
+                        BINARY, ENGINE, ERROR, GENERAL, RELAY, SLOW.
+        """
+        if log_type:
+            self.exec_query("FLUSH {0} LOGS".format(log_type))
+        else:
+            self.exec_query("FLUSH LOGS")
+
     def get_uuid(self):
         """Return the uuid for this server if it is GTID aware.
 
@@ -1377,13 +1523,41 @@ class Server(object):
         """
         from mysql.utilities.common.topology import _GTID_SUBTRACT_TO_EXECUTED
         try:
-            return self.exec_query(
+            result = self.exec_query(
                 _GTID_SUBTRACT_TO_EXECUTED.format(gtid_set)
             )[0][0]
+            # Remove newlines (\n and/or \r) from the GTID set string returned
+            # by the server.
+            return result.replace('\n', '').replace('\r', '')
         except IndexError:
             # If no rows are returned by query then return an empty string.
             return ''
 
+    def inject_empty_trx(self, gtid, gtid_next_automatic=True):
+        """ Inject an empty transaction.
+
+        This method injects an empty transaction on the server for the given
+        GTID.
+
+        Note: SUPER privilege is required for this operation, more precisely
+        to set the GTID_NEXT variable.
+
+        gtid[in]                    GTID for the empty transaction to inject.
+        gtid_next_automatic[in]     Indicate if the GTID_NEXT is set to
+                                    AUTOMATIC after injecting the empty
+                                    transaction. By default True.
+        """
+        self.exec_query("SET GTID_NEXT='{0}'".format(gtid))
+        self.exec_query("BEGIN")
+        self.commit()
+        if gtid_next_automatic:
+            self.exec_query("SET GTID_NEXT='AUTOMATIC'")
+
+    def set_gtid_next_automatic(self):
+        """ Set GTID_NEXT to AUTOMATIC.
+        """
+        self.exec_query("SET GTID_NEXT='AUTOMATIC'")
+
     def checksum_table(self, tbl_name, exec_timeout=0):
         """Compute checksum of specified table (CHECKSUM TABLE tbl_name).
 
@@ -1489,21 +1663,34 @@ class Server(object):
             return False
         return True
 
-    def get_all_databases(self):
+    def get_all_databases(self, ignore_internal_dbs=True):
         """Return a result set containing all databases on the server
         except for internal databases (mysql, INFORMATION_SCHEMA,
-        PERFORMANCE_SCHEMA)
+        PERFORMANCE_SCHEMA).
+
+        Note: New internal database 'sys' added by default for MySQL 5.7.7+.
 
         Returns result set
         """
 
-        _GET_DATABASES = """
-        SELECT SCHEMA_NAME
-        FROM INFORMATION_SCHEMA.SCHEMATA
-        WHERE SCHEMA_NAME != 'INFORMATION_SCHEMA'
-        AND SCHEMA_NAME != 'PERFORMANCE_SCHEMA'
-        AND SCHEMA_NAME != 'mysql'
-        """
+        if ignore_internal_dbs:
+            _GET_DATABASES = """
+            SELECT SCHEMA_NAME
+            FROM INFORMATION_SCHEMA.SCHEMATA
+            WHERE SCHEMA_NAME != 'INFORMATION_SCHEMA'
+            AND SCHEMA_NAME != 'PERFORMANCE_SCHEMA'
+            AND SCHEMA_NAME != 'mysql'
+            """
+            # Starting from MySQL 5.7.7, sys schema is installed by default.
+            if self.check_version_compat(5, 7, 7):
+                _GET_DATABASES = "{0} AND SCHEMA_NAME != 'sys'".format(
+                    _GET_DATABASES)
+        else:
+            _GET_DATABASES = """
+            SELECT SCHEMA_NAME
+            FROM INFORMATION_SCHEMA.SCHEMATA
+            """
+
         return self.exec_query(_GET_DATABASES)
 
     def get_storage_engines(self):
@@ -1610,7 +1797,7 @@ class Server(object):
                 j = create_str.find(" ", i)
                 exist_engine = create_str[i + 7:j]
             else:
-                ## Check if it is a CREATE TABLE LIKE statement
+                # Check if it is a CREATE TABLE LIKE statement
                 is_create_like = (create_str.find("CREATE TABLE {0} LIKE"
                                                   "".format(tbl_name)) == 0)
 
@@ -1837,7 +2024,7 @@ class Server(object):
                            else if False turn autocommit off (set to 0).
         """
         if enable is None:
-             # Switch autocommit value.
+            # Switch autocommit value.
             if self.autocommit is None:
                 # Get autocommit value if unknown
                 self.autocommit_set()
@@ -1939,6 +2126,28 @@ class Server(object):
                     self.grants_enabled = True
         return self.grants_enabled
 
+    def get_server_binlogs_list(self, include_size=False):
+        """Find the binlog file names listed on a server.
+
+        Obtains the binlog file names available on the server by using the
+        'SHOW BINARY LOGS' query at the given server instance and returns these
+        file names as a list.
+
+        include_size[in]  Boolean value to indicate if the returning list shall
+                          include the size of the file.
+
+        Returns a list with the binary logs names available on master.
+        """
+        res = self.exec_query("SHOW BINARY LOGS")
+
+        server_binlogs = []
+        for row in res:
+            if include_size:
+                server_binlogs.append(row)
+            else:
+                server_binlogs.append(row[0])
+        return server_binlogs
+
 
 class QueryKillerThread(threading.Thread):
     """Class to run a thread to kill an executing query.
