@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -84,6 +84,7 @@ def to_sql(obj):
     Returns (string) converted value
     """
     to_sql.__dict__.setdefault('converter', MySQLConverter())
+    obj = to_sql.converter.escape(obj)  # pylint: disable=E1101
     return str(to_sql.converter.quote(obj))  # pylint: disable=E1101
 
 
@@ -191,12 +192,15 @@ def build_pkey_where_clause(table, row):
     if len(pkeys) > 0:
         col_names = table.get_col_names()
         where_str += "WHERE "
+        pkey_cond_lst = []
         for pkey in pkeys:
             key_col = pkey[0]                         # get the column name
             col_data = row[col_names.index(key_col)]  # get column value
             # quote key column with backticks
             q_key_col = quote_with_backticks(key_col)
-            where_str += "%s = %s" % (q_key_col, to_sql(col_data))
+            pkey_cond_lst.append("{0} = {1}".format(q_key_col,
+                                                    to_sql(col_data)))
+        where_str = "{0}{1}".format(where_str, ' AND '.join(pkey_cond_lst))
 
     return where_str
 
@@ -885,7 +889,7 @@ class SQLTransformer(object):
         # Generate DROP foreign key clauses
         for fkey in drop_rows:
             drop_constraints.append("  DROP FOREIGN KEY %s" % fkey[0])
-            #if fkey[0] not in drop_idx_recorded:
+            # if fkey[0] not in drop_idx_recorded:
             #    constraints.append("  DROP INDEX %s" % fkey[0])
 
         # Generate Add foreign key clauses
@@ -985,7 +989,7 @@ class SQLTransformer(object):
 
         # Get the Table instances
         self.dest_tbl = Table(self.destination_db.source, "%s.%s" %
-                             (dest_db, dest_name))
+                              (dest_db, dest_name))
         self.src_tbl = Table(self.source_db.source, "%s.%s" %
                              (src_db, src_name))
 
