@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +36,8 @@ from mysql.utilities.common.tools import execute_script
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.user import User
 from mysql.utilities.common.server import (get_server_state, get_server,
-                                           get_connection_dictionary)
+                                           get_connection_dictionary,
+                                           log_server_version)
 
 
 _HEALTH_COLS = ["host", "port", "role", "state", "gtid_mode", "health"]
@@ -50,13 +51,22 @@ _GTID_WAIT = "SELECT WAIT_UNTIL_SQL_THREAD_AFTER_GTIDS('%s', %s)"
 _GTID_SUBTRACT_TO_EXECUTED = ("SELECT GTID_SUBTRACT('{0}', "
                               "@@GLOBAL.GTID_EXECUTED)")
 
+# TODO: Remove the use of PASSWORD(), depercated from 5.7.6.
 _UPDATE_RPL_USER_QUERY = ('UPDATE mysql.user '
-                          'SET password = PASSWORD("%s")'
-                          'where user ="%s";')
+                          'SET password = PASSWORD("{passwd}")'
+                          'where user ="{user}"')
+# Query for server versions >= 5.7.6.
+_UPDATE_RPL_USER_QUERY_5_7_6 = (
+    "UPDATE mysql.user SET authentication_string = PASSWORD('{passwd}') "
+    "WHERE user = '{user}'")
 
 _SELECT_RPL_USER_PASS_QUERY = ('SELECT user, host, grant_priv, password, '
                                'Repl_slave_priv FROM mysql.user '
-                               'WHERE user ="%s" AND host ="%s";')
+                               'WHERE user ="{user}" AND host ="{host}"')
+# Query for server versions >= 5.7.6.
+_SELECT_RPL_USER_PASS_QUERY_5_7_6 = (
+    "SELECT user, host, grant_priv, authentication_string, "
+    "Repl_slave_priv FROM mysql.user WHERE user ='{user}' AND host ='{host}'")
 
 
 def parse_topology_connections(options, parse_candidates=True):
@@ -260,12 +270,16 @@ class Topology(Replication):
         # attempt to connect to the master
         if master_vals:
             master = get_server('master', master_vals, True, verbose=verbose)
+            if self.logging:
+                log_server_version(master)
 
         for slave_val in slave_vals:
             host = slave_val['host']
             port = slave_val['port']
             try:
                 slave = get_server('slave', slave_val, True, verbose=verbose)
+                if self.logging:
+                    log_server_version(slave)
             except:
                 msg = "Cannot connect to slave %s:%s as user '%s'." % \
                       (host, port, slave_val['user'])
@@ -393,7 +407,8 @@ class Topology(Replication):
                                       'socket': None,
                                       'ssl_ca': self.master.ssl_ca,
                                       'ssl_cert': self.master.ssl_cert,
-                                      'ssl_key': self.master.ssl_key},
+                                      'ssl_key': self.master.ssl_key,
+                                      'ssl': self.master.ssl},
                         'role': slave,
                         'verbose': self.options.get("verbosity", 0) > 0,
                     }
@@ -410,6 +425,8 @@ class Topology(Replication):
                             self._report(msg, logging.INFO, False)
                             if output_log:
                                 print("# {0}".format(msg))
+                            if self.logging:
+                                log_server_version(slave_conn)
                             new_slaves_found = True
                         else:
                             msg = ("Slave skipped (IO not running): "
@@ -737,7 +754,7 @@ class Topology(Replication):
             res = s_candidate.get_rpl_users()
             l = len(res)
             user, host, _ = res[l - 1]
-            #raise
+            # raise
 
         msg = "#   Replication user exists ... %s"
         if user is None or slave.check_rpl_user(user, slave.host) != []:
@@ -1257,48 +1274,59 @@ class Topology(Replication):
               IO_Thread, SQL_Thread, Secs_Behind, Remaining_Delay,
               IO_Error_Num, IO_Error
 
+        Note: The method will return health for the master and slaves or just
+              the slaves if no master is specified. In which case, the master
+              status shall display "no master specified" instead of a status
+              for the connection.
+
         Returns tuple - (columns, rows)
         """
-        assert (self.master is not None), "No master or connection failed."
-
-        # Get master health
-        rpl_health = self.master.check_rpl_health()
-        self._report("# Getting health for master: %s:%s." %
-                     (self.master.host, self.master.port), logging.INFO, False)
-        have_gtid = self.master.supports_gtid()
         rows = []
-        master_data = [
-            self.master.host,
-            self.master.port,
-            "MASTER",
-            get_server_state(self.master, self.master.host, self.pingtime,
-                             self.verbosity > 0),
-            have_gtid,
-            "OK" if rpl_health[0] else ", ".join(rpl_health[1]),
-        ]
-
-        m_status = self.master.get_status()
-        if len(m_status):
-            master_log, master_log_pos = m_status[0][0:2]
-        else:
-            master_log = None
-            master_log_pos = 0
-
         columns = []
         columns.extend(_HEALTH_COLS)
-        # Show additional details if verbosity turned on
         if self.verbosity > 0:
             columns.extend(_HEALTH_DETAIL_COLS)
-            master_data.extend([self.master.get_version(), master_log,
-                                master_log_pos, "", "", "", "", "", "", "", "",
-                                ""])
+        if self.master:
+            # Get master health
+            rpl_health = self.master.check_rpl_health()
+            self._report("# Getting health for master: %s:%s." %
+                         (self.master.host, self.master.port), logging.INFO,
+                         False)
+            have_gtid = self.master.supports_gtid()
+            master_data = [
+                self.master.host,
+                self.master.port,
+                "MASTER",
+                get_server_state(self.master, self.master.host, self.pingtime,
+                                 self.verbosity > 0),
+                have_gtid,
+                "OK" if rpl_health[0] else ", ".join(rpl_health[1]),
+            ]
 
-        rows.append(master_data)
+            m_status = self.master.get_status()
+            if len(m_status):
+                master_log, master_log_pos = m_status[0][0:2]
+            else:
+                master_log = None
+                master_log_pos = 0
+
+            # Show additional details if verbosity turned on
+            if self.verbosity > 0:
+                master_data.extend([self.master.get_version(), master_log,
+                                    master_log_pos, "", "", "", "", "", "",
+                                    "", "", ""])
+
+            rows.append(master_data)
+            if have_gtid == "ON":
+                master_gtids = self.master.exec_query(_GTID_EXECUTED)
+        else:
+            # No master makes these impossible to determine.
+            have_gtid = "OFF"
+            master_log = ""
+            master_log_pos = ""
 
-        slave_rows = []
         # Get the health of the slaves
-        if have_gtid == "ON":
-            master_gtids = self.master.exec_query(_GTID_EXECUTED)
+        slave_rows = []
         for slave_dict in self.slaves:
             host = slave_dict['host']
             port = slave_dict['port']
@@ -1318,11 +1346,13 @@ class Topology(Replication):
                     # Connection failed.
                     rpl_health = (False, ["Slave is not alive."])
                     slave = None
+            elif not self.master:
+                rpl_health = (False, ["No master specified."])
             elif not slave.is_configured_for_master(self.master):
                 rpl_health = (False, ["Slave is not connected to master."])
                 slave = None
 
-            if slave is not None:
+            if self.master and slave is not None:
                 rpl_health = slave.check_rpl_health(self.master,
                                                     master_log, master_log_pos,
                                                     self.max_delay,
@@ -1493,11 +1523,12 @@ class Topology(Replication):
                     res.append((slave.host, slave.port, not_in_set))
             return res
 
-    def check_privileges(self, failover=False):
+    def check_privileges(self, failover=False, skip_master=False):
         """Check privileges for the master and all known servers
 
-        failover[in]   if True, check permissions for switchover and
-                       failover commands. Default is False.
+        failover[in]        if True, check permissions for switchover and
+                            failover commands. Default is False.
+        skip_master[in]     Skip the check for the master.
 
         Returns list - [(user, host)] if not enough permissions,
                        [] if no errors
@@ -1506,14 +1537,23 @@ class Topology(Replication):
         errors = []
 
         # Collect all users first.
-        if self.master is not None:
-            servers.append(self.master)
+        if skip_master:
             for slave_conn in self.slaves:
                 slave = slave_conn['instance']
                 # A slave instance is None if the connection failed during the
                 # creation of the topology. In this case ignore the slave.
                 if slave is not None:
                     servers.append(slave)
+        else:
+            if self.master is not None:
+                servers.append(self.master)
+                for slave_conn in self.slaves:
+                    slave = slave_conn['instance']
+                    # A slave instance is None if the connection failed during
+                    # the creation of the topology. In this case ignore the
+                    # slave.
+                    if slave is not None:
+                        servers.append(slave)
 
         # If candidates were specified, check those too.
         candidates = self.options.get("candidates", None)
@@ -1749,14 +1789,20 @@ class Topology(Replication):
                 # Can't get rpl pass from remote master_repo=file
                 # but it can get the current used hashed to be compared.
                 slave_qry = slave_candidate.exec_query
-                passwd_hash = slave_qry(_SELECT_RPL_USER_PASS_QUERY %
-                                        (user, m_candidate.host))
+                # Use the correct query for server version (changed for 5.7.6)
+                if slave_candidate.check_version_compat(5, 7, 6):
+                    query = _SELECT_RPL_USER_PASS_QUERY_5_7_6
+                else:
+                    query = _SELECT_RPL_USER_PASS_QUERY
+                passwd_hash = slave_qry(query.format(user=user,
+                                                     host=m_candidate.host))
                 # if user does not exist passwd_hash will be an empty query.
                 if passwd_hash:
                     passwd_hash = passwd_hash[0][3]
                 else:
                     passwd_hash = ""
                 # now hash the given rpl password from --rpl-user.
+                # TODO: Remove the use of PASSWORD(), depercated from 5.7.6.
                 rpl_master_pass = slave_qry("SELECT PASSWORD('%s');" %
                                             passwd)
                 rpl_master_pass = rpl_master_pass[0][0]
@@ -1776,7 +1822,12 @@ class Topology(Replication):
                                "password.")
                     self._report("ERROR: %s" % msg, logging.ERROR)
                     return
-            self.master.exec_query(_UPDATE_RPL_USER_QUERY % (passwd, user))
+            # Use the correct query for server (changed for 5.7.6).
+            if self.master.check_version_compat(5, 7, 6):
+                query = _UPDATE_RPL_USER_QUERY_5_7_6
+            else:
+                query = _UPDATE_RPL_USER_QUERY
+            self.master.exec_query(query.format(user=user, passwd=passwd))
 
         if self.verbose:
             self._report("# Creating replication user if it does not exist.")
